# ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
## OUTBREAK PROBABILITY ========================================================
# ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

outbreak_probability <- function(data, 
                                 response = NULL,
                                 samples = NULL,
                                 epi_floor = NULL,
                                 epi_thresh = "quantile",
                                 q = NULL,
                                 sd = NULL,
                                 start_id = NULL) {
  
  # create empty output object
  out <- NULL
  
  # calculate threshold for each month in time series by method
  # (note, this is only informed by years preceding the prediction month)
  
  if (epi_thresh == "quantile"){
    
    # quantile method (with q)
    for (j in sort(unique(data[["year_index"]]))){
      pop <- round(mean(data[data$year_index == j, "population"]))
      add <- data %>%
        select(year_index, month, dir, population) %>%
        filter(year_index < j) %>% 
        group_by(month) %>% 
        summarise(thresh = round(quantile(dir, q) * pop / 10**5, 1),
                  .groups = "drop") %>% 
        mutate(thresh = ifelse((thresh < epi_floor) | is.na(thresh), epi_floor,
                               thresh), year_index = j)
      
      out <- bind_rows(out, add)
    }
  } else if (epi_thresh == "mean"){
    
    # mean plus 1 standard deviation
    for (j in sort(unique(data[["year_index"]]))){
      pop <- round(mean(data[data$year_index == j, "population"]))
      add <- data %>%
        select(year_index, month, dir, population) %>%
        filter(year_index < j) %>% 
        group_by(month) %>% 
        summarise(thresh = round((mean(dir, na.rm = T) +
                                 sd * sd(dir, na.rm = T)) * pop / 10**5, 1),
                  .groups = "drop") %>% 
        mutate(thresh = ifelse((thresh < epi_floor) | is.na(thresh), epi_floor,
                               thresh), year_index = j)
      
      out <- bind_rows(out, add)
    }
  }
  
  # cut down to relevant years
  out <- out[out$year_index >= data[start_id, "year_index"], ]
  data.cut <- data[start_id:nrow(data), ]
  
  # compute outbreak probability
  out <- out %>% 
      bind_cols(list(cases = data.cut[[response]],
                     as.data.frame(samples))) %>%
      rowwise() %>% 
      mutate(pred_mean = mean(c_across(paste0("V", c(1:ncol(samples))))),
             pred_lower = quantile(c_across(paste0("V", c(1:ncol(samples)))),
                                   0.025, na.rm=TRUE),
             pred_upper = quantile(c_across(paste0("V", c(1:ncol(samples)))),
                                   0.975, na.rm=TRUE),
             across(paste0("V", c(1:ncol(samples))), 
                    ~ifelse(.x >= thresh, 1, 0)), 
             obs_outbreak=ifelse(cases>thresh, 1, 0),
             pred_outbreak_prob = sum(c_across(paste0("V", c(1:ncol(samples))))) / 
               ncol(samples)) %>%
      dplyr::select(year_index, month, cases, thresh, pred_mean, 
                    pred_lower, pred_upper, obs_outbreak, pred_outbreak_prob) %>% 
      ungroup()
  
  return(out)
}


## END